# How to Contribute?

## Reporting Bugs
This section guides you through submitting a bug report. Following these guidelines helps maintainers and the community understand your report, reproduce the behavior, and find related reports.

### How Do I Submit A (Good) Bug Report?
Bugs are tracked as GitHub issues. After you've determined which repository your bug is related to, create an issue on that repository and provide the following information by filling in the template.

Explain the problem and include additional details to help maintainers reproduce the problem:

* Use a clear and descriptive title for the issue to identify the problem.
* Describe the exact steps which reproduce the problem in as many details as possible. For example, start by explaining how you started, e.g. which command exactly you used in the terminal. When listing steps, don't just say what you did, but explain how you did it. For example, if you moved the cursor to the end of a line, explain if you used the mouse, or a keyboard shortcut, and if so which one?
* Provide specific examples to demonstrate the steps. Include links to files or GitHub projects, or copy/pasteable snippets, which you use in those examples. 

## Labels Info

We have created labels for each issue for proper ordering and this will help you select an issue you would like to work on.

- The [`documentation` label](https://github.com/ashakhatri007/CSC510_Group25_Project1/projects/1?card_filter_query=label%3Adocumentation) is for problems regarding the entire documentation of this repo.

- The [`enchancement` label](https://github.com/ashakhatri007/CSC510_Group25_Project1/projects/1?card_filter_query=label%3Aenhancement) is for problems regarding any new / existing feature additions to be made in the project.

- The [`bug` label](https://github.com/ashakhatri007/CSC510_Group25_Project1/projects/1?card_filter_query=label%3Abug) is for problems related to any bugs or fixes than need to be taken care of.

- The [`Good first issue` label](https://github.com/ashakhatri007/CSC510_Group25_Project1/projects/1?card_filter_query=label%3A%22good+first+issue%22) Indicates a good issue for first-time contributors.

- The [`tests` label](https://github.com/ashakhatri007/CSC510_Group25_Project1/issues?q=is%3Aissue+is%3Aopen+label%3Atests) Adding missing tests or correcting existing tests

## Suggesting Enhancements

This section guides you through submitting an enhancement suggestion, including completely new features and minor improvements to existing functionality. Following these guidelines helps maintainers and the community understand your suggestion and find related suggestions.

### How Do I Submit A (Good) Enhancement Suggestion?
Enhancement suggestions are tracked as Github issues. After you have determined which repository your enhancement suggestion is related to, create an issue on that repository and provide the following information.

* Use clear and descriptive title for the issue to identify the suggestion.
* Provide a step-by-step description of the suggested enhancement in as many details as possible.
* Provide specific examples to demonstrate the steps. Include copy/pasteable snippets which you use in those examples as Mardown code blocks.
* Describe the current behavior and explain which behavior you expected to see instead and why.
* Include screenshots and animated GIFs which help you demonstrate the steps or point out the part which the suggestion is related to.
* Explain why this enhancement would be useful to most users and is not something that can or should be implemented as a community package.
* List some other text editors or applications where this enhancement exists.
* Specify the name and version of the OS you're using.


## Pull Requests
The process described here has several goals:
* Maintain quality
* Fix problems that are important to users
* Engage the community 
* Enable a sustainable system to review contributions

Please follow these steps to have your contribution considered by the maintainers:
* Follow the styleguides
* After you submit your pull request, verify that all status checks are passing

While the prerequisites above must be satisfied prior to having your pull request reviewed, the reviewer(s) may ask you to complete additional design work, tests, or other changes before your pull request can be ultimately accepted.

## Commit guide

All commit messages must follow the [conventional commit specification](https://www.conventionalcommits.org/en/v1.0.0/). This helps others to easily understand commit logs and allows for automated changelog generation. Here's an example of a commit message that follows the conventional commit spec

    feat: allow provided config object to extend other configs

In general, the messages must follow the following syntax

    <type>[optional scope]: <description>

    [optional body]

    [optional footer(s)]

## Attribution
This Contribution conduct was adapted from [Contributing to Atom][contrib].

[contrib]: https://github.com/atom/atom/blob/master/CONTRIBUTING.md#how-can-i-contribute
