import { API_BASE_URL, ACCESS_TOKEN } from '../constants';

const request = (options) => {
    const headers = new Headers({
        'Content-Type': 'application/json',
    })
    
    if(localStorage.getItem(ACCESS_TOKEN)) {
        headers.append('Authorization', 'Bearer ' + localStorage.getItem(ACCESS_TOKEN))
    }

    const defaults = {headers: headers};
    options = Object.assign({}, defaults, options);

    return fetch(options.url, options)
    .then(response => 
        response.json().then(json => {
            if(!response.ok) {
                return Promise.reject(json);
            }
            return json;
        })
    );
};

export function getCurrentUser() {
    if(!localStorage.getItem(ACCESS_TOKEN)) {
        return Promise.reject("No access token set.");
    }

    return request({
        url: API_BASE_URL + "/user/me",
        method: 'GET'
    });
}

export function login(loginRequest) {
    return request({
        url: API_BASE_URL + "/auth/login",
        method: 'POST',
        body: JSON.stringify(loginRequest)
    });
}

export function signup(signupRequest) {
    return request({
        url: API_BASE_URL + "/auth/signup",
        method: 'POST',
        body: JSON.stringify(signupRequest)
    });
}

export function updateProfile(updateProfileRequest) {
    return request({
        url: API_BASE_URL + "/update",
        method: 'POST',
        body: JSON.stringify(updateProfileRequest)
    });
}

export function fetchExpiredInventoryItems(expiredInventoryItemsRequest) {
    return request({
        url: API_BASE_URL + "/findExpiredInventory",
        method: 'POST',
        body: JSON.stringify(expiredInventoryItemsRequest)
    });
}

export function fetchAboutToExpireInventoryItems(aboutToExpireInventoryItemsRequest) {
    return request({
        url: API_BASE_URL + "/findAboutToExpireInventory",
        method: 'POST',
        body: JSON.stringify(aboutToExpireInventoryItemsRequest)
    });
}

export function fetchLowQuantityInventoryItems(lowQuantityInventoryItemsRequest) {
    return request({
        url: API_BASE_URL + "/findLowQuantityInventoryItems",
        method: 'POST',
        body: JSON.stringify(lowQuantityInventoryItemsRequest)
    });
}